@@ -1,4 +1,4 @@
-# (c) Sound Object Logic 2000-2001
+
 
 use strict;
 
@@ -151,7 +151,7 @@ sub get_importer {
 
 	$import_source = ( "sub { my (\$obj, \$row, \$context) = \@_;"
 			   ."(ref(\$row) eq 'ARRAY') and (\@\$row) or Carp::confess('no row!');\n"
-			   .'# line 1 "foo.pl"'."\n"
+			   ."# line 1 'tangram-$self->{table}-to-$self->{name}.pl'\n"
 			   ."$copy_closures\n$import_source }" );
 
 	print $Tangram::TRACE "Compiling importer for $self->{name}...\n".($Tangram::DEBUG_LEVEL > 1 ? "$import_source\n" : "")."\n"
@@ -210,6 +210,8 @@ use vars qw( %TYPES );
 #   ref      => new Tangram::Ref,
 );
 
+use Scalar::Util qw(reftype);
+
 sub new
 {
     my $pkg = shift;
@@ -220,7 +222,11 @@ sub new
 
     $self->{make_object} ||= sub { shift()->new() };
 
-    $self->{normalize} ||= sub { shift() };
+    $self->{normalize} ||= sub
+	{ my $class = shift;
+	  $class =~ s{::}{_}g;
+	  $class;
+      };
     $self->{class_table} ||= 'OpalClass';
 
     $self->{control} ||= 'Tangram';
@@ -240,7 +246,7 @@ sub new
 
     %$types = ( %TYPES, %$types );
 
-    my @class_list = ref($self->{'classes'}) eq 'HASH' ? %{ $self->{'classes'} } : @{ $self->{'classes'} };
+    my @class_list = reftype($self->{'classes'}) eq 'HASH' ? %{ $self->{'classes'} } : @{ $self->{'classes'} };
     my $class_hash = $self->{'classes'} = {};
 
     bless $class_hash, 'Tangram::ClassHash';
@@ -335,7 +341,8 @@ sub new
     while (my ($class, $classdef) = each %$class_hash)
     {
 		my $root = $class;
-	
+
+		confess "no bases for $root" unless ref $class_hash->{$root}{bases} eq "ARRAY";
 		while (@{$class_hash->{$root}{bases}})
 		{
 			$root = @{$class_hash->{$root}{bases}}[0];
