@@ -1,16 +1,29 @@
-
-
 use strict;
 
 package Tangram::Storage;
 use DBI;
 use Carp;
 use Tangram::Core;
-use Scalar::Util qw(weaken refaddr);
-
+use Carp qw/croak confess/;
 use vars qw( %storage_class );
 
 BEGIN {
+    eval 'use Scalar::Util qw()';
+    if ($@) {
+	*Tangram::refaddr = sub { (shift) + 0 };
+	eval 'use WeakRef';
+	if ($@) {
+	    *Tangram::weaken = sub { };
+	    $Tangram::no_weakrefs = 1;
+	} else {
+	    *Tangram::weaken = \&WeakRef::weaken;
+	    $Tangram::no_weakrefs = 0;
+	}
+    } else {
+	*Tangram::weaken = \&Scalar::Util::weaken;
+	*Tangram::refaddr = \&Scalar::Util::refaddr;
+	$Tangram::no_weakrefs = 0;
+    }
     *pretty = *Tangram::Core::pretty;
 }
 
@@ -162,32 +175,51 @@ sub _open
 		}
 	  }
 	  if ($id) {
-	    $self->{ids}{refaddr($obj)} = $id;
+	    $self->{ids}{Tangram::refaddr($obj)} = $id;
 	  } else {
-	    delete $self->{ids}{refaddr($obj)};
+	    delete $self->{ids}{Tangram::refaddr($obj)};
 	  }
 	};
 
     $self->{get_id} = $schema->{get_id} || sub {
 	  my $obj = shift or warn "no object passed to get_id";
 	  ref $obj or return undef;
-	  my $address = refaddr($obj)
+	  my $address = Tangram::refaddr($obj)
 	      or do { warn "Object $obj has no refaddr(?)";
 		      return undef };
 	  my $id = $self->{ids}{$address};
-	  # refaddr's can be re-used, but weakrefs are magic :-)
-	  if ( $id and !defined $self->{objects}{$id} ) {
-	      delete $self->{ids}{$address};
-	      delete $self->{objects}{$id};
-	      $id = undef;
-	  } elsif ( $id and refaddr($self->{objects}{$id}) != $address ) {
-	      delete $self->{ids}{$address};
-	      $id = undef;
-	  }
 	  if ($Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 2)) {
 		print $Tangram::TRACE "Tangram: $obj is ".($id?"oid $id" : "not in storage")."\n";
 	  }
-	  return $id;
+	  return undef unless $id;
+      if (defined($self->{objects}{$id})) {
+      # This needs to be very paranoid...
+      # $self->{ids} is $address => $id
+      # $self->{objects} is $id => weak $object
+      #
+      # Consider:
+      # 1) Welcome ob1 at XXXX
+      #    $self->{ids}{XXXX} = 1
+      #    $self->{objects}{1} = weak $ob1
+      # 2) $ob1 goes out of scope
+      #    $self->{ids}{XXXX} = 1
+      #    $self->{objects}{1} = undef
+      # 3) Welcome ob 1 at YYYY
+      #    $self->{ids}{XXXX} = 1
+      #    $self->{ids}{YYYY} = 1
+      #    $self->{objects}{1} = weak $ob1
+      # 4) Create new object ob 2 at XXXX, try to lookup if it has an ID
+      #    Find $self->{ids}{XXXX} = 1
+      #    Find defined($self->{objects}{1})
+      #    Eeek - you could get it wrong now! (By returning 1)
+      #    Must check refaddr($self) eq refaddr($self->{objects}{1})
+      #    Not the same object, need to allocate a new ID!
+          return undef unless (Tangram::refaddr($self->{objects}{$id}) eq $address);
+          return $id;
+      }
+	  delete $self->{ids}{$address};
+	  delete $self->{objects}{$id};
+	  return undef;
 	};
 
     return $self;
@@ -667,6 +699,8 @@ sub _insert
 
 	use integer;
 
+    my @unhappy_ending;
+
 	for my $i (0..$#$sths) {
 
 	  if ($Tangram::TRACE) {
@@ -682,13 +716,31 @@ sub _insert
 	  my @args = (map {( ref $_ ? "$_" : $_ )} @state[ @{ $fields[$i] } ]);
 	  #print STDERR "args are: ".Data::Dumper::Dumper(\@args);
 	  #kill 2, $$;
-	  $sth->execute(@args)
-	      or die $dbh->errstr;
+
+      local $@;
+      eval {
+        unless ($sth->execute(@args)) {
+          push @unhappy_ending, $dbh->errstr;
+        }
+      };
+      if ($@) {
+        push @unhappy_ending, $@;
+      }
 
 	  $sth->finish();
 	}
 
-    return $id;
+    if (@unhappy_ending) {
+      $self->{set_id}->($obj, undef);
+
+      # We warn, as we seem to be eating the error message before it gets
+      # printed.
+      warn "Could not completely store object $classId: ".join(", ", @unhappy_ending);
+      die "Could not completely store object $classId: ".join(", ", @unhappy_ending);
+    }
+    else {
+      return $id;
+    }
   }
 
 #############################################################################
@@ -926,13 +978,15 @@ sub load
     return map { scalar $self->load( $_ ) } @_ if wantarray;
 
     my $id = shift;
+    confess("Invalid object-id $id, must match /^\\d+\$/")
+        unless ($id =~ /^\d+$/);
     die if @_;
 
     return $self->{objects}{$id}
       if exists $self->{objects}{$id} && defined $self->{objects}{$id};
 
     my $class = $self->{schema}->classdef( $self->{id2class}{ int(substr($id, -$self->{cid_size})) } );
-
+    confess("Unable to get Tangram class, $id is an invalid object ID?") unless $class;
 	my $row = _fetch_object_state($self, $id, $class);
 
     my $obj = $self->read_object($id, $class->{name}, $row);
@@ -963,7 +1017,7 @@ sub welcome
     my ($self, $obj, $id) = @_;
     $self->{set_id}->($obj, $id);
 
-    weaken( $self->{objects}{$id} = $obj );
+    Tangram::weaken( $self->{objects}{$id} = $obj );
   }
 
 sub goodbye
@@ -1036,7 +1090,7 @@ sub _fetch_object_state
     my $row;
     $sth->execute($self->{export_id}->($id)) &&
 	($row = $sth->fetchrow_arrayref())
-	    or croak "could not find $class->{name} object "
+	    or confess "could not find $class->{name} object "
 		.$self->{export_id}->($id)." (oid $id) in storage";
 
     my $state = [ @$row ] if $row;
