@@ -86,7 +86,7 @@ sub tx_start
     my $storage = shift;
     unless (@{ $storage->{tx} }) {
 	if ( $storage->{no_tx} ) {
-	    $storage->sql_do (q{SELECT GET_LOCK("tx", 10)} ); #})  #cperl-mode--
+	    $storage->sql_do (q{SELECT GET_LOCK('tx', 10)} ); #})  #cperl-mode--
 	}
     }
     $storage->SUPER::tx_start(@_);
@@ -98,7 +98,7 @@ sub tx_commit
     $storage->SUPER::tx_commit(@_);
     unless (@{ $storage->{tx} }) {
 	if ( $storage->{no_tx} ) {
-	    $storage->sql_do(q/SELECT RELEASE_LOCK("tx")/)
+	    $storage->sql_do(q/SELECT RELEASE_LOCK('tx')/)
 	}
     }
   }
@@ -107,7 +107,7 @@ sub tx_rollback
   {
     my $storage = shift;
     if ( $storage->{no_tx} ) {
-	$storage->sql_do(q/SELECT RELEASE_LOCK("tx")/);
+	$storage->sql_do(q/SELECT RELEASE_LOCK('tx')/);
     }
     $storage->SUPER::tx_rollback(@_);
   }
