@@ -73,11 +73,6 @@ $schema =
       Person =>
       {
          abstract => 1,
-       fields =>
-       {
-	string => { colour => undef,
-		  },
-       },
       },
 
       NaturalPerson =>
@@ -228,7 +223,6 @@ $schema =
 	LegalPerson =>
 	{
 	 bases => [ qw( Person ) ],
-	 table => "Person",
 
 	 fields =>
 	 {
